@@ -1,20 +1,23 @@
 use crate::{CONFIG_FILE, SUMMARY_FILE};
 
-use mdbook::errors::Error;
-use std::fs::File;
+use mdbook::errors::*;
+use std::cmp::Ordering;
+use std::fs::{self, File};
 use std::io::Write;
 use std::{
     env,
     path::{Path, PathBuf},
     process::Command,
 };
+use unicase::UniCase;
 use walkdir::WalkDir;
 
 const PAD_SIZE: usize = 4;
 
-/// Search up the filesystem to find a zk.toml file and return it's parent directory.
-pub fn find_zk_root() -> Option<PathBuf> {
-    let mut path: PathBuf = env::current_dir().unwrap().into();
+/// Search up the filesystem to find a `mdzk.toml` file, and return it's parent directory.
+/// This is simply used to find the root of any mdzk vault.
+pub fn find_mdzk_root() -> Option<PathBuf> {
+    let mut path: PathBuf = env::current_dir().unwrap();
     let file = Path::new(CONFIG_FILE);
 
     loop {
@@ -46,19 +49,32 @@ pub fn get_author_name() -> Option<String> {
     }
 }
 
-pub fn update_summary(book_source: &PathBuf) -> Result<(), Error> {
+/// Searches for Markdown-files in the source directory, and updates the summary file
+/// correspondingly. Ignores the summary file itself.
+pub fn update_summary(book_source: &Path) -> Result<(), Error> {
     let summary = WalkDir::new(book_source)
-        .sort_by_file_name()
+        .sort_by_key(|it| {
+            let path_ord = if it.path().is_dir() {
+                Ordering::Less
+            } else {
+                Ordering::Greater
+            };
+
+            (
+                path_ord,
+                UniCase::new(it.file_name().to_string_lossy().to_string()),
+            )
+        })
         .into_iter()
         // remove hidden files
         .filter_entry(|e| {
             !e.file_name()
                 .to_str()
-                .map(|s| s.starts_with("."))
+                .map(|s| s.starts_with('.'))
                 .unwrap_or(false)
         })
         .filter_map(|e| e.ok())
-        // Don't include the book source directory or the SUMMARY.md file
+        // Don't include the book source directory or the summary file
         .filter(|e| e.path() != book_source && e.path() != book_source.join(SUMMARY_FILE))
         .map(|e| {
             let stripped_path = e.path().strip_prefix(&book_source).unwrap();
@@ -84,7 +100,7 @@ pub fn update_summary(book_source: &PathBuf) -> Result<(), Error> {
                 ));
             }
 
-            return None;
+            None
         })
         .filter_map(|e| e)
         .fold(String::new(), |acc, curr| acc + &curr);
@@ -97,11 +113,23 @@ pub fn update_summary(book_source: &PathBuf) -> Result<(), Error> {
     Ok(())
 }
 
+/// String escapes for HTML entity.
 fn escape_special_chars(text: &str) -> String {
-    text.replace(" ", "%20")
-        .replace("<", "&lt;")
-        .replace(">", "&gt;")
-        .replace("'", "&apos;")
-        .replace("\"", "&quot;")
-        .replace("&", "&amp;")
+    text.replace(' ', "%20")
+        .replace('<', "&lt;")
+        .replace('>', "&gt;")
+}
+
+/// Ease-of-use function for creating a file and writing bytes to it
+pub fn write_file(path: &PathBuf, bytes: &[u8]) -> Result<()> {
+    // Create file
+    if let Some(p) = path.parent() {
+        fs::create_dir_all(p)?;
+    }
+    let mut f = File::create(path)?;
+
+    // Write bytes
+    f.write_all(bytes)?;
+
+    Ok(())
 }
